#include "DecodeVedio.h"
#include "spdlog/spdlog.h"
#include "FmtLog/fmtlog.h"



#include <QThread>

extern "C"
{
#include <libavcodec/avcodec.h>
#include <libavformat/avformat.h>
#include <libswscale/swscale.h>
#include <libavutil/imgutils.h>
}


DecodeVedio::DecodeVedio(QThread* thread, QObject* parent) : QObject(parent) , m_thread(thread)
{
    /* 在连接之前调用，移动到新的线程 */
    this->moveToThread(thread);
    m_startThread.setSingleShot(true);
    connect(&m_startThread, &QTimer::timeout, this, &DecodeVedio::do_startDecodeVedio);
    thread->start();
    getHWDecoder();
}

DecodeVedio::~DecodeVedio()
{
    exitThread();
    if(m_thread != nullptr)
    {
        if(m_thread->isRunning())
        {
            m_thread->quit();
            m_thread->wait();
        }
    }

    if(!m_queueImage.isEmpty())
    {
        int size = m_queueImage.count();
        for(int i = 0; i < size; i++)
        {
            auto image = m_queueImage.dequeue();
            if (image)
            {
                delete image;
            }
            
        }
        
    }
}

/* 获取硬件解码器 */
void DecodeVedio::getHWDecoder()
{
    AVHWDeviceType type = AV_HWDEVICE_TYPE_NONE;
    QStringList strTypes;
    while( (type = av_hwdevice_iterate_types(type)) != AV_HWDEVICE_TYPE_NONE)
    {
        m_listHWDeviceType.append(type);
        /* 获取硬件解码器的名称 */
        const char* typeName = av_hwdevice_get_type_name(type);
        if(typeName)
        {
            strTypes.append(QString(typeName));
        }
    }
    FMTLOG_INFO("支持的硬件解码器:");
    for(auto type : strTypes)
    {
        FMTLOG_INFO_NON(" {}",type.toStdString());
    }
    FMTLOG_INFO_NON("\n");
    if(m_listHWDeviceType.isEmpty())
    {
        FMTLOG_WARN("没有找到硬件解码器...");
    }else {
        m_HWDecoder = true;
    }
}

/* 开始解码视频 */
void DecodeVedio::startDecodeVedio()
{
    if(m_isRunning)
    {
        return;
    }
    if(!m_initFFmpeg)
    {
        SPDLOG_WARN("未初始化FFMPEG...");
        return;
    }
    m_isRunning = true;
    // decodeUsingCPU();
    /* 开启定时器，进入槽函数，就进入了新的线程 */
    m_startThread.start(0);
}

/* 停止解码视频，退出工作函数，线程未停止 */
void DecodeVedio::stopDecodeVedio()
{
    if(!m_isRunning)
    {
        return;
    }
    
    m_isRunning = false;
    m_pauseDecode = false;
    /* 唤醒阻塞住的解码线程 */
    m_condQueueNoFull.wakeAll();
    // /* 等待线程执行结束 */
    while(!m_threadStopped)
    {
        /* 睡眠10ms */
        std::this_thread::sleep_for(std::chrono::milliseconds(10));
    }
    
}

/**
 * @brief 设置当前播放位置，单位是微秒
 *        这里需要去掉队列中已有的图片数目对应的时长
 * 
 * @param pos 
 */
void DecodeVedio::setCurrentPos(quint64 pos)
{
    
    /* 暂停解码 */
    m_pauseDecode = true;
    // SPDLOG_DEBUG("setCurrentPos threadID:{}",QThread::currentThreadId());
    while (m_decodeStatus) {
        /* 需要唤醒阻塞住的线程 */
        m_condQueueNoFull.wakeAll();
        std::this_thread::sleep_for(std::chrono::milliseconds(1));
    }
    m_isSeek = true;
    /* 这个是int64最大的正数值 */
    if(pos > 9223372036854775807)
    {
        pos = 0;
    }
    /* 去掉队列中已有的图片数目对应的时长，单位是us */
    qint64 queueNumFPS = m_queueImage.count() * ( 1000.0f / m_fps ) * 1000 ;
    qint64 target_pos = m_startPos + pos - queueNumFPS;
    qint64 end_pos = m_startPos + m_duration;
    /* 检查是否超过视频大小 */
    if(target_pos > end_pos)
    {
        /* 跳转到最后 */
        target_pos = end_pos;
        // SPDLOG_DEBUG("现在的位置:{} 时长:{} 跳转到最后30帧",target_pos, m_duration);
    }
    if(target_pos < m_startPos)
    {
        target_pos = m_startPos;
        // SPDLOG_DEBUG("现在的位置:{} 时长:{} 跳转到开始",target_pos, m_startPos);
    }
    m_currentPos = target_pos;
    SPDLOG_DEBUG("设置播放位置:{}",target_pos);
    /* 第二个参数设置为-1，表示所有流都跳转 */
    int ret = av_seek_frame(m_pFormatContext, -1, target_pos, AVSEEK_FLAG_BACKWARD);
    /* 清空缓存 */
    avcodec_flush_buffers(m_pCodecContext);
    if(ret < 0)
    {
        SPDLOG_WARN("跳转失败...");
    }
    /* 清空队列 */
    m_mutexQueue.lock();
    int size = m_queueImage.count();
    for(int i = 0; i < size; i++)
    {
        auto image = m_queueImage.dequeue();
        if (image)
        {
            delete image;
        }
            
    }
    m_mutexQueue.unlock();
    m_condQueueNoFull.wakeAll();
    SPDLOG_INFO("跳转完成 , queue count {}", m_queueImage.count());
    /* 继续解码 */
    m_pauseDecode = false;
    // SPDLOG_DEBUG("继续解码... {}", m_pauseDecode.load());
}

/* 获取当前播放位置 */
qint64 DecodeVedio::getCurrentPos()
{
    
    return m_currentPos - m_startPos;
}

/* 获取视频时长 */
qint64 DecodeVedio::getDuration()
{
    return m_duration;

}

/* 唤醒队列不满条件变量 */
void DecodeVedio::wakeUpCondQueueNoEmpty()
{
    m_condQueueNoFull.wakeAll();    
}

/**
 * @brief 获取一帧图像，队列为空就返回nullptr，这个函数应该是运行在UI线程中的
 * @warning 传出这个指针后，队列就出队了，内存需要外面获取的实例释放
 * @return QImage* 一帧图像的指针
 */
QImage* DecodeVedio::getOneImage()
{
    QImage* image = m_ringQueue.deQueueBlock();
    return image;
}

/* 获取一帧图像，直到有图像为止 */
QImage* DecodeVedio::getOneImageUntilHave()
{
    QImage* image = nullptr;
    if(m_queueImage.count() == 0)
    {
        m_mutexQueue.lock();
        m_condQueueNoEmpty.wait(&m_mutexQueue);
        image = m_queueImage.dequeue();
        m_mutexQueue.unlock();
    }

    return image;
}

/**
 * @brief 获取每秒的帧数
 * 
 * @return int 正值表示帧数，-1表示未知，-2表示HEVC

 */
int DecodeVedio::getFrameCount()
{
    AVFormatContext *pFormatCtx = NULL;
    avformat_open_input(&pFormatCtx, m_fileName.toStdString().c_str(), NULL, NULL);
    avformat_find_stream_info(pFormatCtx, NULL);
    /* 找到视频流 */
    int videoStreamIndex = -1;
    for (int i = 0; i < pFormatCtx->nb_streams; i++) 
    {
        if (pFormatCtx->streams[i]->codecpar->codec_type == AVMEDIA_TYPE_VIDEO) {
            videoStreamIndex = i;
            break;
        }
    }
    /* 获取视频格式是hevc还是h.264 */
    enum AVCodecID codecID = pFormatCtx->streams[videoStreamIndex]->codecpar->codec_id;
    if (codecID == AV_CODEC_ID_HEVC) 
    {
        SPDLOG_DEBUG("视频格式是HEVC");
        m_fps = -2;
        // AVPacket packet;
        // int frameCount = 0;
        // double timeBase = av_q2d(pFormatCtx->streams[videoStreamIndex]->time_base);
        // double duration = 2.0;  // Duration in seconds

        // while (av_read_frame(pFormatCtx, &packet) >= 0) 
        // {
        //     if (packet.stream_index == videoStreamIndex) 
        //     {
        //         double packetTime = packet.pts * timeBase;
        //         if (packetTime <= duration) {
        //             frameCount++;
        //         } else {
        //             break;
        //         }
        //     }
        //     av_packet_unref(&packet);
        // }
        // fps = frameCount / duration;
    }
    else if(codecID == AV_CODEC_ID_H264)
    {
        SPDLOG_DEBUG("视频格式是H.264");
        /* 获取到分子和分母，一除就是帧率 */
        AVRational frameRate = pFormatCtx->streams[videoStreamIndex]->avg_frame_rate;
        m_fps = frameRate.num*1.0 / frameRate.den;
        // SPDLOG_DEBUG("分子:{} 分母:{} 帧率:{}",frameRate.num, frameRate.den, fps);
    }else
    {
        m_fps = 30;
    }

    avformat_close_input(&pFormatCtx);

    return m_fps;
}
/**
 * @brief 设置图像宽度和高度
 *        注意：目前不好用，建议使用Qt的图片缩放
 * 
 * @param width 
 * @param height 
 */
// void DecodeVedio::setVideoSize(int width, int height)
// {
//     m_width = width;
//     m_height = height;

//     if(m_sws_ctx != nullptr)
//     {
//         /* 先暂停解码器 */
//         pauseDecode();
//         sws_freeContext(m_sws_ctx);
//     }
//     /* 初始化Sws Context，这是转换规则，转换成RGB */
//     m_sws_ctx = sws_getContext( m_pCodecContext->width, m_pCodecContext->height, m_pCodecContext->pix_fmt,    /* 原图像大小和格式 */
//                                 m_width, m_height, AV_PIX_FMT_RGB24,      /* 目标图像的大小和格式 */
//                                 SWS_BICUBIC, 
//                                 nullptr, 
//                                 nullptr, 
//                                 nullptr);
// }


/* 初始化硬件解码器 */
void DecodeVedio::initHWDecoder(const AVCodec* codec)
{

}


/* 打开视频，同时初始化解码器） */
void DecodeVedio::openVedio(const QString& fileName)
{
    if(fileName.isEmpty())
    {
        SPDLOG_WARN("文件名为空...");
        return;
    }
    m_fileName = fileName;
    if(m_initFFmpeg)
    {
        freeAll();
    }

    /* 清空队列 */
    if(m_ringQueue.QueueSize() > 0)
    {
        for(int i = 0; i < m_ringQueue.QueueSize(); i++)
        {
            auto image = m_ringQueue.deQueueBlock();
            if (image)
            {
                delete image;
            }
        }
        m_ringQueue.clearQueue();
    }
    m_ringQueue.setQueueSize(30);

    SPDLOG_DEBUG("开始初始化FFMPEG");
    AVDictionary* dict = nullptr;
    av_dict_set(&dict, "rtsp_transport", "tcp", 0);      // 设置rtsp流使用tcp打开，如果打开失败错误信息为【Error number -135 occurred】可以切换（UDP、tcp、udp_multicast、http），比如vlc推流就需要使用udp打开
    av_dict_set(&dict, "max_delay", "3", 0);             // 设置最大复用或解复用延迟（以微秒为单位）。当通过【UDP】 接收数据时，解复用器尝试重新排序接收到的数据包（因为它们可能无序到达，或者数据包可能完全丢失）。这可以通过将最大解复用延迟设置为零（通过max_delayAVFormatContext 字段）来禁用。
    av_dict_set(&dict, "timeout", "1000000", 0);         // 以微秒为单位设置套接字 TCP I/O 超时，如果等待时间过短，也可能会还没连接就返回了。

    /************ 存储文件格式信息 ************/
    /* 打开文件，读取视频文件的头信息，放在第一个参数的结构体中 */
    int ret = avformat_open_input(  &m_pFormatContext,                  /* 解封装上下文 */
                                    m_fileName.toStdString().data(),    /* 打开视频地址 */
                                    nullptr,                            /* 这个参数强制使用特定的输入格式，可以设置为null */
                                    &dict);                             /* 参数设置 */
    if(ret != 0)
    {
        SPDLOG_WARN("打开视频文件错误，错误代码:{}",ret);
        freeAll();
        return;
    }
    /* 释放参数 */
    if(dict != nullptr)
    {
        av_dict_free(&dict);
    }

    /************ 找到视频流 ************/
    /* 检查视频容器内部的流信息，将所有流存储到了pFormatContext->streams中
     * 查找到视频流，并获取视频时长相关的信息 */
    ret = 0;
    ret = avformat_find_stream_info(m_pFormatContext, nullptr);
    if(ret < 0)
    {
        SPDLOG_WARN("获取视频流错误，错误代码:{}",ret);
        freeAll();
        return;
    }
    m_duration = m_pFormatContext->duration;
    m_currentPos = 0;
    m_startPos = m_pFormatContext->start_time;
    // SPDLOG_DEBUG("开始时间:{} 时长:{}",m_startPos, m_duration);
    
    /* 一个调试函数，将流信息输出到控制台 */
    av_dump_format(m_pFormatContext, 0, m_fileName.toStdString().c_str(), 0);

    /* 找到视频流 */
    m_videoStream = av_find_best_stream(m_pFormatContext, AVMEDIA_TYPE_VIDEO, -1, -1, nullptr, 0);
    if(m_videoStream < 0)
    {
        SPDLOG_WARN("没有找到视频流");
        freeAll();
        return;
    }
    SPDLOG_INFO("找到视频流");
    /* 获取视频流的编解码信息，主要是分辨率等信息 */
    AVStream *pStream = m_pFormatContext->streams[m_videoStream];   /* 获取视频流 */
    AVCodecParameters* pCodecParams = pStream->codecpar;            /* 获取视频流的编解码信息 */
    SPDLOG_INFO("获取视频流参数成功!");
    /* 获取视频相关信息 */
    m_width = m_srcWidth = pCodecParams->width;
    m_height = m_srcHeight = pCodecParams->height;
    m_fps = rationalToDouble(&pStream->avg_frame_rate);
    m_totalFrame = m_pFormatContext->streams[m_videoStream]->nb_frames;


    /************ 查找并设置解码器 ************/
    /* 找到解码器 */
    const AVCodec* pCodec = avcodec_find_decoder(pCodecParams->codec_id);
    if(pCodec == nullptr)
    {
        SPDLOG_WARN("没有找到解码器");
        freeAll();
        return;
    }
    SPDLOG_INFO("找到解码器:{}",pCodec->name);
    /* 获取视频信息的上下文，先分配空间，后面记得释放空间 */
    m_pCodecContext = avcodec_alloc_context3(pCodec);
    /* 将视频流中的编码器参数拷贝下来，这个函数不是线程安全的 */
    if(avcodec_parameters_to_context(m_pCodecContext, pCodecParams) != 0)
    {
        SPDLOG_WARN("复制上下文错误");
        freeAll();
        return;
    }
    SPDLOG_INFO("设置解码器参数成功!");
    // m_pCodecContext->flags2 |= AV_CODEC_FLAG2_FAST;  /* 使用快速解码（允许使用不符合规范的解码） */
    m_pCodecContext->thread_count = 8;      /* 使用8线程解码 */

    /* 初始化硬件解码器 */
    if(m_HWDecoder)
    {
        initHWDecoder(pCodec);
    }

    /* 打开解码器，（初始化解码器上下文，如果调用了avcodec_alloc_context3，第二个参数可以设置为nullptr） */
    if(avcodec_open2(m_pCodecContext, nullptr, nullptr) < 0)
    {
        SPDLOG_ERROR("打开解码器错误");
        freeAll();
        return;
    }
    SPDLOG_DEBUG("打开解码器成功!");

    /************ 初始化数据包 ************/
    m_packet = av_packet_alloc();
    av_new_packet(m_packet, m_pCodecContext->width * m_pCodecContext->height);

    /* 创建两个pFrame，一个存放原始数据，一个存放转换后的RGB数据 */
    m_pFrameSRC = av_frame_alloc();
    if(m_pFrameSRC == nullptr)
    {
        SPDLOG_ERROR("创建pFrame错误");
        freeAll();
        return;
    }
    m_pFrameRGB = av_frame_alloc();
    if(m_pFrameRGB == nullptr)
    {
        SPDLOG_ERROR("创建pFrameRGB错误");
        freeAll();
        return;
    }
    m_pFrameHW = av_frame_alloc();
    if(m_pFrameHW == nullptr)
    {
        SPDLOG_ERROR("创建pFrameHW错误");
        freeAll();
        return;
    }

    if(m_buffer != nullptr)
    {
        av_free(m_buffer);
        m_buffer = nullptr;
    }
    
    /* 分配图像空间 */
    int numBytes = av_image_get_buffer_size(AV_PIX_FMT_RGBA, m_pCodecContext->width, m_pCodecContext->height, 1);
    /* 这里多分配了一些空间，防止某些视频图像在使用sws_scale()拷贝后超出数组长度 */
    m_buffer = (uint8_t *)av_malloc(numBytes + 1000);

    m_initFFmpeg = true;
    SPDLOG_INFO("FFMPEG初始化完成!");
    SPDLOG_INFO("视频宽度:{} 高度:{} 帧率:{} 总时长:{} 总帧数:{}",m_width, m_height, m_fps, m_duration, m_totalFrame);
}


/**
 * @brief 软解码线程，使用CPU解码，使用环境队列存储解码的数据
 * 
 */
void DecodeVedio::decodeUsingCPU()
{
    /******** 初始化局部变量 ********/
    bool isEnd = false;
    int ret = 0;
    int retFrame = 0;
    int retPacket = 0;
    m_pauseDecode = false;
    m_decodeStatus = true;
    
    /* 设置环形队列的大小 */
    m_ringQueue.setQueueSize(30);

    /* 开始解码 */
    SPDLOG_DEBUG("开始解码...");
    while(m_isRunning)
    {
        /******** 读取数据包 av_read_frame ********/
        int retRead = av_read_frame(m_pFormatContext, m_packet);
        if(retRead == AVERROR_EOF)
        {
            /* 读取到末尾后，需要传入一个空的packet，才能读取到最后几帧 */
            avcodec_send_packet(m_pCodecContext, m_packet);
        }
        else if(retRead < 0)
        {
            SPDLOG_ERROR("读取帧错误...");
            break;
        }
        else
        {
            if(m_packet->stream_index == m_videoStream)
            {
            #if 1
                /* 计算当前帧时间，两种方法，第一种适用于所有场景，但是存在一定的误差 */
                m_packet->pts = qRound64(m_packet->pts * (1000 * rationalToDouble(&m_pFormatContext->streams[m_videoStream]->time_base)));
                m_packet->dts = qRound64(m_packet->dts * (1000 * rationalToDouble(&m_pFormatContext->streams[m_videoStream]->time_base)));
            #else
                /* 适用于本地视频，直接计算本地视频文件的每一帧时间 */
                m_currentFrame++;
                // m_packet->pts = qRound64(m_currentFrame * (qreal(m_duration) / m_totalFrame));
            #endif

                /* 将数据传给解码器 */
                int ret = avcodec_send_packet(m_pCodecContext, m_packet);
                if(ret < 0)
                {
                    SPDLOG_ERROR("发送数据包错误...");
                    av_packet_unref(m_packet);
                    continue;
                }
            }else {
                SPDLOG_INFO("不是视频流。");
                av_packet_unref(m_packet);
                continue;
            }
        }

        SPDLOG_DEBUG("读取到数据包packet，pts:{}",m_packet->pts);
        /* 解码packet包的内容，一个packet包内可能包含好多帧视频 */
        while(m_isRunning)
        {
            /* 读取出解码器返回的帧 avcodec_receive_frame */
            int ret = avcodec_receive_frame(m_pCodecContext, m_pFrameSRC);
            if(ret == AVERROR_EOF)
            {
                SPDLOG_INFO("解码完成...");
                isEnd = true;
                break;
            }
            else if (ret == AVERROR(EAGAIN))
            {
                /* packet中的内容无法解码成一帧，需要更多的输入包，可能已经取出了几帧，也肯能就不够一帧
                 * 这时候就需要往解码器送数据包了 */
                SPDLOG_WARN("packet无法解码成一帧，需要更多的输入包");
                av_frame_unref(m_pFrameSRC);
                break;
            }
            else if(ret < 0)
            {
                av_frame_unref(m_pFrameSRC);
                if(retRead < 0)
                {
                    SPDLOG_ERROR("读取错误，错误值:{}",ret);
                    break;
                }
            }
            /* 解码成功，获取到帧数 */
            m_pts = m_pFrameSRC->pts;
            /* 转换解码后的帧格式，转换成RGBA格式，Qt可以识别 */
            if(m_sws_ctx == nullptr)
            {
                /* 选择在这里创建，为了兼容硬件解码，硬件解码出来的格式可能和解码器传出的不一样 */
                m_sws_ctx = sws_getCachedContext(m_sws_ctx, 
                                                m_pFrameSRC->width, m_pFrameSRC->height,      /* 原图像大小和格式 */
                                                m_pCodecContext->pix_fmt,                       /* 输入图像的像素格式 */
                                                m_width, m_height,                          /* 目标图像的大小 */
                                                AV_PIX_FMT_RGBA,                            /* 目标图像的格式 */
                                                SWS_BILINEAR,                               /* 图像缩放算法，双线性 */
                                                nullptr,                                    /* 输入图像的滤波器信息，不需要传NULL */
                                                nullptr,                                    /* 输出图像的滤波器信息，不需要传NULL */
                                                nullptr);                                   /* 特定缩放算法需要的参数，不需要传NULL */
                if(m_sws_ctx == nullptr)
                {
                    SPDLOG_ERROR("创建SwsContext错误...");
                    av_packet_free(&m_packet);
                    av_frame_free(&m_pFrameSRC);
                    av_frame_free(&m_pFrameRGB);
                    return;
                }
                SPDLOG_INFO("创建SwsContext成功...");
            }
            /* 转换成RGBA格式 */
            uint8_t* data[1] = { m_buffer };
            int lines[4];
            /* 使用像素格式pix_fmt和宽度填充图像的平面线条的大小（一行大小？） */
            av_image_fill_linesizes(lines, AV_PIX_FMT_RGBA, m_pFrameSRC->width);

            sws_scale(  m_sws_ctx,              /* 缩放的上下文 */
                        m_pFrameSRC->data,      /* 源图像数组 */
                        m_pFrameSRC->linesize,  /* 包含源图像每个平面步幅的数组 */
                        0,                      /* 开始位置 */
                        m_pFrameSRC->height,    /* 行数 */
                        data,                   /* 目标图像数组 */
                        lines); /* 目标图像行数 */
            if(m_buffer != nullptr)
            {
                /* 将数据拷贝到QImage中 */
                auto image = new QImage(m_pFrameRGB->data[0], m_width, m_height, QImage::Format_RGBA8888);
                m_ringQueue.enQueueBlock(image);
                av_frame_unref(m_pFrameRGB);
                SPDLOG_DEBUG("一帧视频入队");
            }
            av_frame_unref(m_pFrameSRC);
        }
        av_packet_unref(m_packet);    /* 释放数据包，引用计数-1，为0时释放空间 */
        if(isEnd)
        {
            break;
        }

        // AVFrame* pFrameTemp = m_pFrameSRC;
        // if(m_pFrameSRC->data[0] == nullptr)
        // {
        //     /* 使用的是硬件解码器 */
        //     pFrameTemp = m_pFrameHW;
        //     /* 将数据从GPU拷贝到内存中 */
            
        //     break;
        // }
    }
    /* 释放空间 */
    av_packet_free(&m_packet);
    av_free(m_pFrameSRC);
    av_free(m_pFrameRGB);
}

/* 退出线程，将所有可能暂停线程运行的条件全部唤醒 */
void DecodeVedio::exitThread()
{
    if(m_isRunning)
    {
        m_isRunning = false;
    }
    m_pauseDecode = false;
    m_condQueueNoFull.wakeAll();
}

/* 暂停解码，会阻塞到线程暂停为止 */
void DecodeVedio::pauseDecode()
{
    m_pauseDecode = true;
    while (m_decodeStatus) {
        /* 需要唤醒阻塞住的线程 */
        m_condQueueNoFull.wakeAll();
        std::this_thread::sleep_for(std::chrono::milliseconds(1));
    }
}

/**
 * @brief          将AVRational转换为double，用于计算帧率
 * @param rational
 * @return
 */
qreal DecodeVedio::rationalToDouble(AVRational* rational)
{
    qreal frameRate = (rational->den == 0) ? 0 : (qreal(rational->num) / rational->den);
    return frameRate;
}


/* 开启解码 */
void DecodeVedio::do_startDecodeVedio()
{
    SPDLOG_DEBUG("线程ID:{}",QThread::currentThreadId());
    // if(!m_initFFmpeg)
    // {
    //     initFFmpeg();
    // }
    m_isRunning = true;
    m_threadStopped = false;
    m_pauseDecode = false;
    /* 进入解码，直到播放完成或者手动退出 */
    decodeUsingCPU();
    SPDLOG_TRACE("Decode播放完成...");

}

/* 释放所有资源 */
void DecodeVedio::freeAll()
{
    if(m_sws_ctx)
    {
        sws_freeContext(m_sws_ctx);
        m_sws_ctx = nullptr;
    }
    if(m_pFrameRGB)
    {
        av_frame_free(&m_pFrameRGB);
    }
    if(m_pFrameSRC)
    {
        av_frame_free(&m_pFrameSRC);
    }
    if(m_pFrameHW)
    {
        av_frame_free(&m_pFrameHW);
    }
    if(m_packet)
    {
        av_packet_free(&m_packet);
    }
    if(m_pCodecContext)
    {
        avcodec_free_context(&m_pCodecContext);
    }
    if(m_pFormatContext)
    {
        avformat_close_input(&m_pFormatContext);
    }



    if(!m_queueImage.isEmpty())
    {
        int size = m_queueImage.count();
        for(int i = 0; i < size; i++)
        {
            auto image = m_queueImage.dequeue();
            if (image)
            {
                delete image;
            }
        }
    }

}

